package com.ether;

import java.util.Formatter;
import java.util.Locale;

@SuppressWarnings("serial")
public class FormattedException extends Exception
{
    public FormattedException(final Object... parameters) {
	this(Locale.getDefault(), DEFAULT_MESSAGE, DEFAULT_THROWABLE, parameters);
    }

    public FormattedException(final Locale locale, final Object... parameters) {
	this(locale, DEFAULT_MESSAGE, DEFAULT_THROWABLE, parameters);
    }

    public FormattedException(final String message, final Object... parameters) {
	this(Locale.getDefault(), message, DEFAULT_THROWABLE, parameters);
    }

    public FormattedException(final Locale locale, final String message, final Object... parameters) {
	this(locale, message, null, parameters);
    }

    public FormattedException(final Throwable throwable, final Object... parameters) {
	this(Locale.getDefault(), throwable.getMessage(), throwable, parameters);
    }

    public FormattedException(final String message, final Throwable throwable, final Object... parameters) {
	this(Locale.getDefault(), message, throwable, parameters);
    }

    public FormattedException(final Locale locale, final String message, final Throwable throwable, final Object... parameters) {
	super(format(locale, message, parameters), throwable);
    }

    private static String format(final Locale locale, final String message, final Object[] parameters)
    {
	final StringBuilder stringBuilder = new StringBuilder();
	final Formatter formatter = new Formatter(stringBuilder);
	formatter.format(locale, message, parameters);
	return stringBuilder.toString();
    }

    /**
     * Prefixes the message with code if not null.
     */
    public static String prefix(final Enum<? extends Code> code, final String message)
    {
	final StringBuffer stringBuffer = new StringBuffer();

	if (code != null)
	{
	    stringBuffer.append(code.toString());
	    stringBuffer.append(" - ");
	}

	stringBuffer.append(message);
	return stringBuffer.toString();
    }

    /**
     * The error code, it identifies the type of error. 
     * The derived classes must creates their own code as enum and implements this interface. 
     * If the subclass is not an enum it won't be recognized by the constructor.
     */
    public static interface Code
    {
    }

    public static String DEFAULT_MESSAGE = "no message";
    public static String DEFAULT_THROWABLE = null;
}
